#include <algorithm>
#include <iostream>
#include <memory>

struct node {
	int value;
	std::unique_ptr<node> next;
	node* prev;

	node(int v, std::unique_ptr<node> n, node* p)
	  : value(v), next(std::move(n)), prev(p) {}

	~node() {
		// just to show when elements get deallocated
		std::cout << "destroyed: " << value << "\n";
	}
};

struct list {
	std::unique_ptr<node> first;
	node* last = nullptr;

	void append(int v) {
		auto new_last = std::make_unique<node>(v, nullptr, last);
		// try writing the following without code duplication
		if (last) {
			last->next = std::move(new_last);
			last = last->next.get();
		} else {
			first = std::move(new_last);
			last = first.get();
		}
	}

	void prepend(int v) {
		node* orig = first.get();

		first =	std::make_unique<node>(v, std::move(first), nullptr);

		// try writing the following without code duplication
		if (orig) {
			orig->prev = first.get();
		} else {
			last = first.get();
		}
	}

	void print(bool rev = false) const {
		const node* curr = rev ? last : first.get();
		while (curr) {
			std::cout << curr->value << ' ';
			curr = rev ? curr->prev : curr->next.get();
		}
		std::cout << '\n';
	}

	~list() {
		std::cout << "list destroyed\n";
	}
};

int main() {
	list lst;
	for (int i : { 1, 2, 3, 4, 5 }) {
		lst.append(i);
	}
	lst.print();
	lst.print(/* rev = */ true);
}
