#include <algorithm>
#include <iostream>
#include <memory>

struct node {
	int value;
	std::unique_ptr<node> next;
	node* prev;

	node(int v, std::unique_ptr<node> n, node* p)
	  : value(v), next(std::move(n)), prev(p) {}

	~node() {
		// just to show when elements get deallocated
		std::cout << "destroyed: " << value << "\n";
	}
};

struct list {
	std::unique_ptr<node> first;
	node* last = nullptr;

	// append/prepend, now with less code duplication

	void append(int v) {
		auto& after = last ? last->next : first;
		after = std::make_unique<node>(v, nullptr, last);
		last = after.get();
	}

	void prepend(int v) {
		auto& before = first ? first->prev : last;
		first =	std::make_unique<node>(v, std::move(first), nullptr);
		before = first.get();
	}

	void print(bool rev = false) const {
		const node* curr = rev ? last : first.get();
		while (curr) {
			std::cout << curr->value << ' ';
			curr = rev ? curr->prev : curr->next.get();
		}
		std::cout << '\n';
	}

	~list() {
		std::cout << "list destroyed\n";
	}
};

int main() {
	list lst;
	for (int i : { 1, 2, 3, 4, 5 }) {
		lst.append(i);
	}
	lst.print();
	lst.print(/* rev = */ true);
}
